using Interpolations

"""
data, header = read_initial_guess(;newinit=true)
returns
time, p, p_dot, theta, theta_dot, a_ref, a_ref_dot
"""
function read_initial_guess(;newinit=true)
    local names, data, header
    if newinit
        data, header = readdlm("initial_guess.csv", ' ', Float64, header=true)
    else
        #This is the case for old jmodelica initial guess
        data, header = readdlm("initial_guess_jm.csv", ' ', Float64, header=true)
        data = data[:,[1,7,8,9,10,11,12]]
        header = header[[1,7,8,9,10,11,12]]
    end
    return data, vec(header)
end


function set_guess!(n, t, states, controls, tf)
    x = n.r.ocp.x
    u = n.r.ocp.u
    N = size(x,1)           # Number of time steps in optimization problem
    nx = size(states,2)     # Number of states
    nu = size(controls,2)   # Number of controls
    @assert nx == size(x,2) # Guess and problem have same number of states
    @assert nu == size(u,2) # Guess and problem have same number of inputs
    @assert size(states,1) == size(states,1) == length(t) # Guess has right lengts
    # Set finial time guess
    setvalue(n.ocp.tf, tf)
    for i = 1:nx # For each state
        # Interpolate guess based on time t
        xi = interpolate((t,), states[:,i], Gridded(Linear()))
        for j = 1:N                     # For each time point
            tj = tf/(N-1)*(j-1)             # Time of gridpoint j
            setvalue(x[j,i], xi(tj))    # Set guess
        end
    end
    # Set control guess
    for i = 1:nu
        ui = interpolate((t,), controls[:,i], Gridded(Linear()))
        for j = 1:N
            tj = tf/(N-1)*(j-1)
            setvalue(u[j,i], ui(tj))
        end
    end
    return
end

function randomInit!(n,N)
    u = n.r.ocp.u
    x = n.r.ocp.x
    for i = 1:N
        setvalue(u[i,1], (rand()-0.5)*100)
        setvalue(x[i,1], rand()*0.5-0.25)
        setvalue(x[i,2], rand()-0.5)
        setvalue(x[i,3], rand()-0.5)
        setvalue(x[i,4], rand()-0.5)
        setvalue(x[i,5], (rand()-0.5)*5)
    end
end

# # OLD CODE FOR SETTING INITIAL GUESS MANUALLY
# initialdata, header, datainfo = read_initial_guess()
# # Initial guess
# t_guess = initialdata[:,1]
# states_guess = initialdata[:,7:11]
# controls_guess = initialdata[:,12:12]
# tf_guess = initialdata[end,1]
# set_guess!(n, t_guess, states_guess, controls_guess, tf_guess)
# #
# # p_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,7:7]), Gridded(Linear()))
# # dp_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,8:8]), Gridded(Linear()))
# # theta_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,9:9]), Gridded(Linear()))
# # dtheta_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,10:10]), Gridded(Linear()))
# # a_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,11:11]), Gridded(Linear()))
# # da_g = interpolate((vec(initialdata[:,1]),), vec(initialdata[:,12:12]), Gridded(Linear()))
# # x = n.r.ocp.x
# # u = n.r.ocp.u
# # tf_0 = initialdata[end,1]
# # setvalue(n.ocp.tf, tf_0)
# # for i = 1:(N+1)
# #     t = tf_0/N*(i-1)
# #     setvalue(x[i,1],      p_g(t))
# #     setvalue(x[i,2],     dp_g(t))
# #     setvalue(x[i,3],  theta_g(t))
# #     setvalue(x[i,4], dtheta_g(t))
# #     setvalue(x[i,5],      a_g(t))
# #     setvalue(u[i,1],     da_g(t))
# # end
